/*----------------------------------------------------------*
   SAM7S/SE/X 簡易ブートローダ Ver 2.7
                                       Last Edit '16 8/16
   Copyright (c) BestTechnology CO.,LTD. 2005-2016
 *----------------------------------------------------------*/

/*---------------------------------------------------------------------------*
 ■概要

  ATMEL AT91SAM7 シリーズの内蔵フラッシュを書き換えるアプリケーション。
  とは言ってもインテリジェンス性よりも、自前のアプリケーションを簡便に
  FLASH へ書き込んで実行させる事を目的とした。
  なお、実行できる自前アプリケーションは若干のトリックが必要なため、
  GCC Developer Lite では適切なコンパイルオプションを適用すること。

 ■コンパイル時の注意事項

  1.コンパイル可能な環境
   最新版 GCC Developer Lite が SAM7S/SE/X ターゲットファイルを含む形で
   インストールされていること。
  2.コンパイル及び書き込み方法
   付属の MAKE_AND_WRITE.BAT を実行し、メッセージに従って適宜入力。
   コンパイルが成功すると JTAG (openOCD) にて書き込みを行う。

 ■ユーザプログラムに関して

  1.ブートアドレス
   0x1002000
  2.メモリマップ
   0x0にマッピングされるメモリはRAMとし(REMAP必須)、割り込みベクタ
   を別途登録する事(GDLを使う限り意識する必要は無い)。
  3.ペリフェラル
   ローダからアプリケーションに遷移する際にCKGR及びRSTCはリセット
   直後の状態に戻されないので、ブート時にそれを期待した構造を避ける。

 ■更新履歴

  Ver.2.7 2016/08/16
     各ターゲットライブラリ更新に伴う再構築
     ソフトディレイのシーケンスを変更
     XMODEM-1Kに未対応部分(主にUSBのバッファ処理)を修正
     DeinitではRSTCによるペリフェラル初期化に変更
     SAM7SE追加
     'x'(ソフトリセット)後のRTTの挙動が不安定なためdeinitでは無駄ループに変更
     TODO:初期条件によってFLASHの書き込みに失敗する事がある

  Ver.2.6 2014/01/27
     SAM7S/X用シリアル通信ライブラリ更新に合わせて修正

  Ver.2.5 2013/08/06
     USBのポートオープン判定はWindows限定だったため廃止
     コマンドモードへ入るために'!'を入力するのが野暮ったかったので、
     ハードリセットの検出ないしはユーザプログラムが書かれていない時
     のみコマンドモードへ遷移する様に変更

  Ver.2.4 2011/07/14
     USBのデフォルト化をもうちょっと真面目に

  Ver.2.3 2010/05/01
     RTTの挙動に納得がいったので元に戻す。

  Ver.2.2 2010/04/16
     RTTの挙動が怪しげに思えてきたので変更。
     ちょっとだけメモリに余裕が出てきたので_USE_SECURITYBITを活性化。

  Ver.2.12 2010/03/31
     SAM7S/Xターゲットのシリアルライブラリ更新に伴いコード修正。

  Ver.2.11 2010/03/07
     GCCの更新に伴い512kデバイスのコードサイズが8kを超えてしまうので、
     少しだけコードサイズが削減されるようマクロを調整。

  Ver.2.1  2010/02/09
     PCからのOPEN/CLOSE判定を若干厳密に。案の定時折違うものがあるの
     で。
     メモリが厳しくなってきたので出来るだけ余計な機能を削除。
     コンパイルにはGCC Developer Lite 2.3.2.0以上を使用の事。

  Ver.2.0  2009/05/12
     またコンパイラの都合からまたまたバイナリサイズが大きくなってし
     まったので、苦渋の決断でメモリの読み出し機能を殺し、ブートアド
     レスを全て 0x1002000 共通とした。
     旧来のブートローダが仕込まれたターゲットはブートローダをV2.0に
     しなければ、最新GDL2.1.1.47以降に対応できない。
     おまけでCDCEMUのポートオープンを判断して、USB経由でターミナル
     プログラムでポートを開くと起動メッセージを送出する様にした。

  Ver.1.9  2009/03/27
     AT91SAM7Xリリース版のUSBプルアップ制御がリセット信号に同期する
     ため、RSTCを制御してリセット端子を制御する事に。
     またコンパイラの都合からまたまたバイナリサイズが大きくなってし
     まったので、アプリケーションのブートアドレスを以下の通りとする。
       AT91SAM7S32        0x1002000
       AT91SAM7S64～256   0x1002100
       AT91SAM7S512       0x1002200
       AT91SAM7X128～256  0x1002100
       AT91SAM7X512       0x1002200
     これに伴い、従来のGCC Developer Liteにおいてブートローダを前
     提としたアプリケーションは最新版にて再構築が必要となる。
     また、旧来のブートローダが仕込まれたターゲットはブートローダ
     をV1.9にしないと、最新GCC Developer Liteでは対応できない。
     今後どこまでずれてゆくのやら・・・。

  Ver.1.8  2009/01/10
     アプリケーションジャンプ時のペリフェラル初期化順序とUSBプルアッ
     プ解除のタイミングを変更。
     512kデバイスのブートアドレスを0x102300から0x102100に変更。

  Ver.1.7  2008/11/24
     512kデバイスサポート。
     UDPによるシリアルエミュレーションポートサポート。
     初回のPC接続時にドライバをインストールする必要があるが、識別
     に数秒のタイムアウトを設けてあるため、インストール中にタイム
     アウトしてしまうのはご愛敬。
     SAM-BAスイッチの誤操作によるブートローダの誤消去を抑止すべく
     セキュリティビットの書き換え機能をサポート。ONにしてしまうと
     JTAGやSAM-BAでの書き換えが行えなくなるが、ERASE端子の操作での
     みNVMを初期状態に戻せる。

  Ver.1.6  2008/04/18
     Ver.1.6から適用するgccの吐くコードサイズが若干上がったため、
     渋々8kbyteを本コードのエリアとする。
     よってダウンロード対象のアプリケーションをコンパイルする際に
     GCC Developer Lite 2.0.1.2以降を使用しないとブートのブートア
     ドレスが異なるため動かない。
     TeraTermのXMODEMでダウンロードが開始されない場合があったはず
     だが、なぜかこの版から動くようになる。

  Ver.1.51 2007/12/12
     SAM7Sの内蔵FLASHの状況に個体差がある様で、書き込みに失敗する
     個体の確率が数%と高いため、ウェイトの設定を長くした。
     この版を流用したOEM向け装置が多いが、v1.6以降の版で改善される
     ネタがあるので無視してもOK。

  Ver.1.5  2007/08/28
     DBGU(ポーリング)にてダウンロードの失敗確率が上がっていたので
     微調整したが失敗。
     DBGUポートを使う場合はDBGU2を選択すべし。
     訳もなく入っていた処理を削除したら書き込み速度UP。

  Ver.1.4  2007/05/10
     RS485(半二重)通信モードを追加。

  Ver.1.3  2007/04/24
     XMODEMのCRCに対応。
     ブートアドレスを0x101B00に決める。

  Ver.1.2
     SAM7Xに対応。
     ついでにターゲットを変更してもコンパイルオプションを選択し直
     してコンパイルすれば自動的に反映されるように変更。

  Ver.1.1
     NVMの書き換えやFSRを見れる様にする。

  Ver.1.0
     初版。
     SAM-BAの操作が煩雑で個体差によっては識別しにくいケースが見受け
     られ始めたので作る事を決める。
 *---------------------------------------------------------------------------*/

#if defined (__AT91SAM7S32__) |  defined (__AT91SAM7S64__) | defined (__AT91SAM7S128__) | defined (__AT91SAM7S256__) | defined (__AT91SAM7S512__)
#include  <AT91SAM7S.h>
#include  <AT91SAM7S_BITDEFINE.h>
#elif defined (__AT91SAM7X128__) | defined (__AT91SAM7X256__) | defined (__AT91SAM7X512__)
#include  <AT91SAM7X.h>
#include  <AT91SAM7X_BITDEFINE.h>
#elif defined (__AT91SAM7SE32__) | defined (__AT91SAM7SE256__) | defined (__AT91SAM7SE512__)
#include  <AT91SAM7SE.h>
#include  <AT91SAM7SE_BITDEFINE.h>
#endif
#include  <us.h>
#include  <vic.h>
#include  <stdbool.h>
#include  <stdlib.h>
#include  <stdint.h>
#include  <string.h>
#include  <ctype.h>

// セキュリティビットのコマンド対応 (宣言・非宣言で選択)
#define _USE_SECURITYBIT

// XMODEMによるダウンロード対応 (宣言・非宣言で選択)
//#define _USE_DOWNLOAD

// パケットサイズ (128か1024のいずれか)
//#define _XMODEM_PACKET_SIZE (128)
#define _XMODEM_PACKET_SIZE (1024)

// 使用コンソールポート (以下のいずれか)
//#define __DBGU__    // DBGUポーリング
//#define __DBGU2__   // DBGU割り込み
//#define __US0__     // US0割り込み
//#define __US1__     // US1割り込み
//#define __USB__     // USBポーリング

// US0/US1時のRS485対応 (宣言・非宣言で選択)
//#define __RS485__

#if defined (__AT91SAM7SE32__) | defined (__AT91SAM7SE256__) | defined (__AT91SAM7SE512__)
#define _MON_LED  ((uint32_t) 1 << 1) // PA0
#elif defined (__AT91SAM7S32__) |  defined (__AT91SAM7S64__) | defined (__AT91SAM7S128__) | defined (__AT91SAM7S256__) | defined (__AT91SAM7S512__)
#define _MON_LED  ((uint32_t) 1 << 0) // PA1
#endif

//************************************************
// XMODEM
//************************************************
#define _SOH  (0x01)
#define _STX  (0x02)
#define _EOT  (0x04)
#define _ACK  (0x06)
#define _NAK  (0x15)
#define _CAN  (0x18)
#define _XYN  (_NAK)
#define _XYC  ('C')
#define _XYG  ('G')

//************************************************
// 使用するコンソールポート
//************************************************
#define _BAUDRATE           (115200)

#if defined (__DBGU__)
#define us_rx_buff  dbgu_rx_buff
#define us_tx_buff  dbgu_tx_buff
#define us_rx_purge dbgu_rx_purge
#define us_putc     dbgu_putc
#define us_getc     dbgu_getc
#define us_puts     dbgu_puts
#define us_putsb    dbgu_putsb
#define us_gets     dbgu_sys_gets
#define us_printf   dbgu_printf
#define PORTNAME    "DBGU"
#elif defined (__DBGU2__)
#define us_rx_buff  dbgu_int_rx_buff
#define us_tx_buff  dbgu_int_tx_buff
#define us_rx_purge dbgu_int_rx_purge
#define us_tx_purge dbgu_int_tx_purge
#define us_putc     dbgu_int_putc
#define us_getc     dbgu_int_getc
#define us_puts     dbgu_int_puts
#define us_gets     dbgu_int_sys_gets
#define us_putsb    dbgu_int_putsb
#define us_printf   dbgu_int_printf
#define PORTNAME    "DBGU"
#elif defined (__US0__)
#define us_rx_buff  us0_int_rx_buff
#define us_tx_buff  us0_int_tx_buff
#define us_rx_purge us0_int_rx_purge
#define us_tx_purge us0_int_tx_purge
#define us_putc     us0_int_putc
#define us_getc     us0_int_getc
#define us_puts     us0_int_puts
#define us_putsb    us0_int_putsb
#define us_gets     us0_int_sys_gets
#define us_printf   us0_int_printf
#define PORTNAME    "US0"
#elif defined (__US1__)
#define us_rx_buff  us1_int_rx_buff
#define us_tx_buff  us1_int_tx_buff
#define us_rx_purge us1_int_rx_purge
#define us_tx_purge us1_int_tx_purge
#define us_putc     us1_int_putc
#define us_getc     us1_int_getc
#define us_puts     us1_int_puts
#define us_gets     us1_int_sys_gets
#define us_putsb    us1_int_putsb
#define us_printf   us1_int_printf
#define PORTNAME    "US1"
#elif defined (__USB__)
#if defined (__AT91SAM7S32__)
#error "NOT SUPPORT UDP"
#else
#include "cdc_enumerate.h"
// "cdc_enumerate.c"
#define us_rx_buff  udps_rx_buff
#define us_rx_purge udps_rx_purge
#define us_putc     udps_putc
#define us_getc     udps_getc
#define us_puts     udps_puts
#define us_putsb    udps_putsb
#define PORTNAME    "USB"

#if defined (__AT91SAM7S64__) | defined (__AT91SAM7S128__) | defined (__AT91SAM7S256__) | defined (__AT91SAM7S512__)
#define _PULLUP_EXT_GPIO
#define _UDP_PU   ((uint32_t) 1 << 16) // PA16 -> UDP UpllUp
#elif defined (__AT91SAM7X128__) | defined (__AT91SAM7X256__) | defined (__AT91SAM7X512__)
#define _PULLUP_EXT_NRST
#elif defined (__AT91SAM7SE32__) | defined (__AT91SAM7SE256__) | defined (__AT91SAM7SE512__)
#define _PULLUP_INTERNAL
#else
#error "NOT SUPPORT UDP"
#endif

#endif
#else
#error "NOT DEFINED PORT"
#endif

#if defined (__DBGU__) | defined (__DBGU2__) | defined (__US0__) | defined (__US1__)
uint8_t txb[_XMODEM_PACKET_SIZE + 20], rxb[_XMODEM_PACKET_SIZE + 20];
#elif defined (__USB__)
uint8_t rxb[_XMODEM_PACKET_SIZE + 20];
struct _AT91S_CDC pCDC;
#endif

//************************************************
// フラッシュメモリの情報
//************************************************
#if defined (__AT91SAM7S32__)
#define FLASH_START_OFFSET  (0x2000U)
#define FLASH_ADDRESS       (FLASH_BASE_ADDRESS + FLASH_START_OFFSET)
#define FLASH_BYTE_SIZE     (FLASH_PAGE_NB * FLASH_PAGE_SIZE_BYTE - FLASH_START_OFFSET)
#elif (__AT91SAM7S64__) | defined (__AT91SAM7S128__) | defined (__AT91SAM7S256__) | defined (__AT91SAM7X128__) | defined (__AT91SAM7X256__)
//#define FLASH_START_OFFSET  (0x2100U)
#define FLASH_START_OFFSET  (0x2000U)
#define FLASH_ADDRESS       (FLASH_BASE_ADDRESS + FLASH_START_OFFSET)
#define FLASH_BYTE_SIZE     (FLASH_PAGE_NB * FLASH_PAGE_SIZE_BYTE - FLASH_START_OFFSET)
#elif defined (__AT91SAM7S512__) | defined (__AT91SAM7X512__)
//#define FLASH_START_OFFSET  (0x2200U)
#define FLASH_START_OFFSET  (0x2000U)
#define FLASH_ADDRESS       (FLASH_BASE_ADDRESS + FLASH_START_OFFSET)
#define FLASH_ADDRESS1      (FLASH_BASE_ADDRESS +FLASH_PAGE_NB * FLASH_PAGE_SIZE_BYTE)
#define FLASH_BYTE_SIZE     (FLASH_PAGE_NB * FLASH_PAGE_SIZE_BYTE * 2 - FLASH_START_OFFSET)
#elif defined (__AT91SAM7SE32__) | defined (__AT91SAM7SE256__) | defined (__AT91SAM7SE512__)
#define FLASH_START_OFFSET  (0x2000U)
#define FLASH_ADDRESS       (FLASH_BASE_ADDRESS + FLASH_START_OFFSET)
#define FLASH_ADDRESS1      (FLASH_BASE_ADDRESS +FLASH_PAGE_NB * FLASH_PAGE_SIZE_BYTE)
#define FLASH_BYTE_SIZE     (FLASH_PAGE_NB * FLASH_PAGE_SIZE_BYTE * 2 - FLASH_START_OFFSET)
#endif

#define _MC_CORRECT_KEY     (0x5A000000U)                 // (MC) Correct Protect Key
#define _IFLASH             ((char *)FLASH_BASE_ADDRESS)  // Internal ROM base address

const char *mess[] = {
  // 0
  "\n--< TIMSAM7 "PORTNAME" Ver.2.7 by BestTechnology >--",
  // 1
#ifdef _USE_DOWNLOAD
  "\n [w]UPLOAD [r]DOWNLOAD [g]BOOT [x]REBOOT [d]DUMP\n"
  " [i]FLASH INFO [e]ERASE [n]SET NVM BIT [c]CLER NVM BIT\n"
  " [l]SET LOCK BIT [u]CLEAR LOCK BIT",
#else
  "\n [w]UPLOAD [g]BOOT [x]REBOOT [d]DUMP\n"
  " [i]FLASH INFO [e]ERASE [n]SET NVM BIT [c]CLER NVM BIT\n"
  " [l]SET LOCK BIT [u]CLEAR LOCK BIT",
#endif
};

/*======================================================================*/
/* UDP SERIAL EMULATION                                                 */
/*======================================================================*/
#if defined (__USB__)
typedef struct {
  struct {
    uint8_t *buf;
    uint16_t wp;
    uint16_t rp;
    uint16_t maxlen;
  } rx;
} TFIFO;

TFIFO FIFO;

void rx_int (void) {
  static uint8_t data[_XMODEM_PACKET_SIZE + 20];

  if (pCDC.IsConfigured (&pCDC)) {
    while (pCDC.RxBuff (&pCDC)) {
      int l = pCDC.Read (&pCDC, data, _XMODEM_PACKET_SIZE + 20);
      for (int i = 0; i < l; i++) {
        int t = ((int)FIFO.rx.wp + 1) % FIFO.rx.maxlen;
        FIFO.rx.buf[FIFO.rx.wp] = data[i];
        if (t != FIFO.rx.rp) FIFO.rx.wp = t;
      }
    }
  } else FIFO.rx.wp = FIFO.rx.rp = 0;
}

int udps_rx_buff (void) {
  int s;

  rx_int();
  if ((s = ((int)FIFO.rx.wp - (int)FIFO.rx.rp)) < 0) s += FIFO.rx.maxlen;
  rx_int();
  return s;
}

void udps_putc (char ch) {
  rx_int();
  pCDC.Write (&pCDC, (uint8_t *)&ch, 1);
  rx_int();
}

char udps_getc (void) {
  char c;

  while (FIFO.rx.rp == FIFO.rx.wp) rx_int();
  c = FIFO.rx.buf[FIFO.rx.rp++];
  FIFO.rx.rp %= FIFO.rx.maxlen;
  rx_int();
  return c;
}

void udps_puts (const char *s) {
  static char tmp[2000];
  char *p = tmp;

  while ((*p++ = *s)) if (*s++ == '\n') * p++ = '\r';
  *p = '\0';
  rx_int();
  pCDC.Write (&pCDC, (uint8_t *)tmp, strlen (tmp));
  rx_int();
}

int udps_putsb (const uint8_t *s, int len) {
  rx_int();
  pCDC.Write (&pCDC, s, len);
  rx_int();
  return len;
}

void udps_rx_purge (void) {
  while (udps_rx_buff()) {
    udps_getc();
    rx_int();
  }
}

#if defined (__AT91SAM7X128__) | defined (__AT91SAM7X256__) | defined (__AT91SAM7X512__)
void udps_reset (void) {
#define _RESET_LENGTH   (0xe)
  // Execute reset
  uint32_t rmr = RSTC.MR;

  rmr &= ~ ((0xff << 24) | _RSTC_ERSTL);
  rmr |= (_RESET_LENGTH << 8) & _RSTC_ERSTL;
  RSTC.MR = rmr | (0xA5 << 24);
  RSTC.CR = _RSTC_EXTRST | (0xA5 << 24);
  // Wait for end hardware reset
  while (! (RSTC.SR & _RSTC_NRSTL));
}
#endif
#endif

/*======================================================================*/
/* WAIT                                                                 */
/*======================================================================*/
#if 0
#define _USTICK  (30UL)
#define _MSTICK  (30518UL)

static inline void init_rtt (uint32_t *oldreg) {
  static bool noninit = true;

  if (noninit) {
    noninit = false;
    RTT.MR = _RTT_RTTRST | 1;
  }
  if (*oldreg == 0) *oldreg = RTT.VR;
}

static uint32_t get_cyccnt_diff (uint32_t *oldreg) {
  uint32_t tick_now = RTT.VR;
  uint32_t result;

  if (tick_now >= *oldreg) result = tick_now - *oldreg;
  else result = (UINT32_MAX - *oldreg) + tick_now;
  *oldreg = tick_now;
  return result;
}

uint32_t GetTick_ms (void) {
  static uint32_t mymstick = 0;
  static uint32_t oldVR = 0;

  init_rtt (&oldVR);
  mymstick += (get_cyccnt_diff (&oldVR));
  return mymstick / _USTICK;
}
#else
#define _USTICK  (30UL)
#define _MSTICK  (30518UL)

static inline void init_rtt (uint32_t *oldreg) {
  static bool noninit = true;

  if (noninit) {
    noninit = false;
    RTT.MR = _RTT_RTTRST | 1;
    uint32_t c = RTT.VR ;
    while (RTT.VR == c) ;
  }
  if (*oldreg == 0) *oldreg = RTT.VR;
}

static uint32_t get_cyccnt_diff (uint32_t *oldreg) {
  uint32_t tick_now = RTT.VR;
  uint32_t result;

  if (tick_now >= *oldreg) result = tick_now - *oldreg;
  else result = (UINT32_MAX - *oldreg) + tick_now;
  *oldreg = tick_now;
  return result;
}

uint32_t GetTick_ms (void) {
  static uint32_t mymstick = 0;
  static uint32_t oldVR = 0;

  init_rtt (&oldVR);
  mymstick += (get_cyccnt_diff (&oldVR));
  return mymstick / _USTICK;
}
#endif
bool wait (int ms, bool rx) {
  uint32_t et = GetTick_ms() + ms;

  while (et > GetTick_ms()) if (rx) if (us_rx_buff()) return true;
  return false;
}

/*======================================================================*/
/* INIT                                                                 */
/*======================================================================*/
void FL_FlashInit (void);

void DeInit (void) {
#if defined (__USB__)
#if defined (_PULLUP_EXT_GPIO)
  PIOA.SODR = _UDP_PU;
//  wait (1000, false);
  for(volatile int i=0;i<3000000;i++);
#elif defined (_PULLUP_EXT_NRST)
  udps_reset();
#elif defined (_PULLUP_INTERNAL)
  UDP.TXVC = _UDP_TXVDIS;
//  wait (1000, false);
  for(volatile int i=0;i<3000000;i++);
#endif
#endif

  disableIRQ();
  RSTC.CR = 0xA5000000 | _RSTC_PERRST;
  // AICはリセット対象外
  AIC.IDCR = (uint32_t) - 1;
  AIC.ICCR = (uint32_t) - 1;
//  AIC.EOICR = 0;
  AIC.SPU = 0;
  AIC.DCR = 0;
  AIC.FFDR = (uint32_t) - 1;
  for (int i = 0; i < 32; i++) {
    AIC.SMR[i] = 0;
    AIC.SVR[i] = 0;
  }
  PMC.PCDR = PMC.PCSR;
}

void Init (void) {
  FL_FlashInit();

#if defined (_UDP_PU) & defined (_MON_LED)
  PMC.PCER = 1 << PIOA_PID;
  PIOA.SODR = _MON_LED;
  PIOA.CODR = _UDP_PU;
  PIOA.MDDR = _UDP_PU | _MON_LED;
  PIOA.PUDR = _UDP_PU | _MON_LED;
  PIOA.OER = _UDP_PU | _MON_LED;
  PIOA.PER = _UDP_PU | _MON_LED;
#elif defined (_UDP_PU) & !defined (_MON_LED)
  PMC.PCER = 1 << PIOA_PID;
  PIOA.CODR = _UDP_PU;
  PIOA.MDDR = PIOA.PUDR = PIOA.OER = PIOA.PER = _UDP_PU;
#elif !defined (_UDP_PU) & defined (_MON_LED)
  PMC.PCER = 1 << PIOA_PID;
  PIOA.SODR = _MON_LED;
  PIOA.MDDR = PIOA.PUDR = PIOA.OER = PIOA.PER = _MON_LED;
#endif

#if defined (__DBGU__)  /////////// DBGU
  dbgu_init (_BAUDRATE, txb, sizeof (txb), rxb, sizeof (rxb));
#elif defined (__DBGU2__)
  dbgu_int_init (_BAUDRATE, txb, sizeof (txb), rxb, sizeof (rxb));
  enableIRQ();
#elif defined (__US0__) /////////// US0
  us0_int_init (_BAUDRATE, txb, sizeof (txb), rxb, sizeof (rxb));
#if defined (__RS485__)
  us0_int_rs485 = true;
#endif
  enableIRQ();
#elif defined (__US1__) /////////// US1
  us1_int_init (_BAUDRATE, txb, sizeof (txb), rxb, sizeof (rxb));
#if defined (__RS485__)
  us1_int_rs485 = true;
#endif
  enableIRQ();
#elif defined (__USB__) /////////// USB
  FIFO.rx.buf = rxb;
  FIFO.rx.maxlen = sizeof (rxb);
  FIFO.rx.wp = FIFO.rx.rp = 0;

  // Specific Chip USB Initialisation
  // Enables the 48MHz USB clock UDPCK and System Peripheral USB Clock
  PMC.SCER = _PMC_UDP;
  PMC.PCER = 1 << UDP_PID;
#if defined (_PULLUP_EXT_GPIO)
  wait (1000, false);
#elif defined (_PULLUP_EXT_NRST)
  UDP.TXVC = 0;
  udps_reset();
#elif defined (_PULLUP_INTERNAL)
  UDP.TXVC = _UDP_PUON;
#endif
  // CDC Open by structure initialization
  AT91F_CDC_Open (&pCDC);
#endif
}

/*======================================================================*/
/* FLASH CTRL                                                           */
/*======================================================================*/
int __attribute__ ((section (".ramfunc"))) FL_FlashReady (void) {
  uint32_t status;

  do {
    status = MC.EFC0.FSR;
  } while ((status & _MC_FRDY) == 0);
  return status;
}

#if defined (__AT91SAM7S512__) | defined (__AT91SAM7X512__) | defined (__AT91SAM7SE512__)
int __attribute__ ((section (".ramfunc"))) FL_FlashReady1 (void) {
  uint32_t status;

  do {
    status = MC.EFC1.FSR;
  } while ((status & _MC_FRDY) == 0);
  return status;
}
#endif

#define fmcn  ((F_CPU) / 100000)

void __attribute__ ((section (".ramfunc"))) FL_FlashInit (void) {
  MC.EFC0.FMR = (_MC_FMCN & (fmcn << 16)) | _MC_FWS_1FWS;
#if defined (__AT91SAM7S512__) | defined (__AT91SAM7X512__) | defined (__AT91SAM7SE512__)
  MC.EFC1.FMR = (_MC_FMCN & (fmcn << 16)) | _MC_FWS_1FWS;
#endif
}

void __attribute__ ((section (".ramfunc"))) FL_NVMSet (uint8_t NVM_Number) {
  uint32_t  _cpsr;

  if (! (((MC.EFC0.FSR >> 8) & 0xFF) & (1 << NVM_Number))) {
    _cpsr = disableIRQ();
    FL_FlashInit();
    MC.EFC0.FCR = _MC_CORRECT_KEY | _MC_FCMD_SET_GP_NVM | (_MC_PAGEN & (NVM_Number << 8));
    FL_FlashReady();
    restoreIRQ (_cpsr);
  }
}

void __attribute__ ((section (".ramfunc"))) FL_NVMClear (uint8_t NVM_Number) {
  uint32_t  _cpsr;

  if ((((MC.EFC0.FSR >> 8) & 0xFF) & (1 << NVM_Number))) {
    _cpsr = disableIRQ();
    FL_FlashInit();
    MC.EFC0.FCR = _MC_CORRECT_KEY | _MC_FCMD_CLR_GP_NVM | (_MC_PAGEN & (NVM_Number << 8));
    FL_FlashReady();
    restoreIRQ (_cpsr);
  }
}

#if defined(_USE_SECURITYBIT)
__attribute__ ((section (".ramfunc"))) void FL_SetSecurity (void) {
  uint32_t  _cpsr;

  _cpsr = disableIRQ();
  FL_FlashInit();
  MC.EFC0.FCR = _MC_CORRECT_KEY | _MC_FCMD_SET_SECURITY;
  FL_FlashReady();
  restoreIRQ (_cpsr);
}
#endif

bool __attribute__ ((section (".ramfunc"))) FL_PageLocked (uint32_t page) {
#if defined (__AT91SAM7S512__) | defined (__AT91SAM7X512__) | defined (__AT91SAM7SE512__)
  if (page < FLASH_PAGE_NB)
    return ((MC.EFC0.FSR & ((1 << (page / FLASH_SECTOR_PAGE)) << 16)) != 0);
  else
    return ((MC.EFC1.FSR & ((1 << ((page % FLASH_PAGE_NB) / FLASH_SECTOR_PAGE)) << 16)) != 0);
#else
  return ((MC.EFC0.FSR & ((1 << (page / FLASH_SECTOR_PAGE)) << 16)) != 0);
#endif
}

void __attribute__ ((section (".ramfunc"))) FL_Flashlock (uint32_t page) {
  uint32_t  _cpsr;

  if (!FL_PageLocked (page)) {
    _cpsr = disableIRQ();
    FL_FlashInit();
#if defined (__AT91SAM7S512__) | defined (__AT91SAM7X512__) | defined (__AT91SAM7SE512__)
    if (page < FLASH_PAGE_NB) {
      MC.EFC0.FCR = _MC_CORRECT_KEY | _MC_FCMD_LOCK | (_MC_PAGEN & (page << 8));
      FL_FlashReady();
    } else {
      MC.EFC1.FCR = _MC_CORRECT_KEY | _MC_FCMD_LOCK | (_MC_PAGEN & ((page % FLASH_PAGE_NB) << 8));
      FL_FlashReady1();
    }
#else
    MC.EFC0.FCR = _MC_CORRECT_KEY | _MC_FCMD_LOCK | (_MC_PAGEN & (page << 8));
    FL_FlashReady();
#endif
    restoreIRQ (_cpsr);
  }
}

void __attribute__ ((section (".ramfunc"))) FL_FlashUnlock (uint32_t page) {
  uint32_t  _cpsr;

  if (FL_PageLocked (page)) {
    _cpsr = disableIRQ();
    FL_FlashInit();
#if defined (__AT91SAM7S512__) | defined (__AT91SAM7X512__) | defined (__AT91SAM7SE512__)
    if (page < FLASH_PAGE_NB) {
      MC.EFC0.FCR = _MC_CORRECT_KEY | _MC_FCMD_UNLOCK | (_MC_PAGEN & (page << 8));
      FL_FlashReady();
    } else {
      MC.EFC1.FCR = _MC_CORRECT_KEY | _MC_FCMD_UNLOCK | (_MC_PAGEN & ((page % FLASH_PAGE_NB) << 8));
      FL_FlashReady1();
    }
#else
    MC.EFC0.FCR = _MC_CORRECT_KEY | _MC_FCMD_UNLOCK | (_MC_PAGEN & (page << 8));
    FL_FlashReady();
#endif
    restoreIRQ (_cpsr);
  }
}

bool __attribute__ ((section (".ramfunc"))) FL_FlashWrite (uint32_t adr, int sz, uint32_t *buf) {
  uint32_t  _cpsr;
  uint32_t  i, page, status;
  uint32_t   *Flash = (uint32_t *)adr;

  page = ((adr - (uint32_t)FLASH_BASE_ADDRESS) / FLASH_PAGE_SIZE_BYTE);
#if defined (__AT91SAM7S512__) | defined (__AT91SAM7X512__) | defined (__AT91SAM7SE512__)
  if (page >= (FLASH_PAGE_NB << 1)) return false;
#else
  if (page >= FLASH_PAGE_NB) return false;
#endif
  for (i = 0; (i < FLASH_PAGE_SIZE_BYTE) & (sz > 0); i++, sz -= 4) * Flash++ = *buf++;
  _cpsr = disableIRQ();
  if (FL_PageLocked (page)) {
#if defined (__AT91SAM7S512__) | defined (__AT91SAM7X512__) | defined (__AT91SAM7SE512__)
    if (page < FLASH_PAGE_NB) {
      MC.EFC0.FCR = _MC_CORRECT_KEY | _MC_FCMD_UNLOCK | (_MC_PAGEN & (page << 8));
      FL_FlashReady();
    } else {
      MC.EFC1.FCR = _MC_CORRECT_KEY | _MC_FCMD_UNLOCK | (_MC_PAGEN & ((page % FLASH_PAGE_NB) << 8));
      FL_FlashReady1();
    }
#else
    MC.EFC0.FCR = _MC_CORRECT_KEY | _MC_FCMD_UNLOCK | (_MC_PAGEN & (page << 8));
    FL_FlashReady();
#endif
  }
#if defined (__AT91SAM7S512__) | defined (__AT91SAM7X512__) | defined (__AT91SAM7SE512__)
  if (page < FLASH_PAGE_NB) {
    MC.EFC0.FCR = _MC_CORRECT_KEY | _MC_FCMD_START_PROG | (_MC_PAGEN & (page << 8));
    status = FL_FlashReady();
  } else {
    MC.EFC1.FCR = _MC_CORRECT_KEY | _MC_FCMD_START_PROG | (_MC_PAGEN & ((page % FLASH_PAGE_NB) << 8));
    status = FL_FlashReady1();
  }
#else
  MC.EFC0.FCR = _MC_CORRECT_KEY | _MC_FCMD_START_PROG | (_MC_PAGEN & (page << 8));
  status = FL_FlashReady();
#endif
  restoreIRQ (_cpsr);

  if ((status & (_MC_PROGE | _MC_LOCKE)) != 0) return false;
  return true;
}

bool FL_WriteFlash (uint32_t Flash_Address, int size, uint32_t *buff) {
  int       next;
  bool      status;
  uint32_t  dest, *src;

  FL_FlashInit();
  dest = Flash_Address;
  src = buff;
  status = true;
  while (status & (size > 0)) {
    if (size <= FLASH_PAGE_SIZE_BYTE) next = size;
    else                              next = FLASH_PAGE_SIZE_BYTE;
    status = FL_FlashWrite (dest, next, src);
    size -= next;
    src += FLASH_PAGE_SIZE_BYTE / 4;
    dest += FLASH_PAGE_SIZE_BYTE;
  }
  return status;
}

bool EraseFlashAll (void) {
  bool    noterased;
  uint8_t buf[FLASH_PAGE_SIZE_BYTE];

  memset (buf, 0xff, FLASH_PAGE_SIZE_BYTE);
  for (uint32_t i = FLASH_ADDRESS; i < FLASH_ADDRESS + FLASH_BYTE_SIZE; i += FLASH_PAGE_SIZE) {
    for (uint32_t j = 0; j < FLASH_PAGE_SIZE_BYTE >> 2; j += 4)
      if ((noterased = (* (uint32_t *) (i + j)) != 0xffffffff)) break;
    if (noterased) if (!FL_WriteFlash (i, FLASH_PAGE_SIZE_BYTE, (void *)buf)) return false;
    if (i % 12288 == 0) us_putc ('.');
  }
  return true;
}

/*======================================================================*/
/* BINARY CHARACTOR CONVERT                                             */
/*======================================================================*/
uint8_t ResetType (void) {
  return (RSTC.SR & _RSTC_RSTTYP) >> 8;
}

char *I2H (int32_t v, short len, char *s) {
  const char hex_code[] = "0123456789ABCDEF";

  for (int i = 0; i < len; i++) s[i] = '0';
  s[len] = 0;
  do {
    s[--len] = hex_code[v & 0xf];
    v >>= 4;
  } while (len && v);
  return s;
}

int64_t H2I (char *s) {
  int64_t i, n = 0;

  if (!*s) return -1;
  for (i = 0; *s; i++, s++) {
    if ('0' <= *s && *s <= '9') {
      n <<= 4;
      n |= *s - '0';
    } else if ('a' <= *s && *s <= 'f') {
      n <<= 4;
      n |= *s - ('a' - 10);
    } else if ('A' <= *s && *s <= 'F') {
      n <<= 4;
      n |= *s - ('A' - 10);
    }
  }
  return n;
}

char *I2D (long v, short len, char *s) {
  bool f = v < 0;

  v = labs (v);
  memset (s, ' ', len);
  s[len] = 0;
  do {
    s[--len] = (v % 10) + '0';
    v /= 10;
  } while (len && v);
  if (f && (len > 0)) s[--len] = '-';
  return s;
}

bool AreYouSure (const char *s) {
  char c;

  us_puts (s);
  us_puts ("\n ARE YOU SURE? (Y/N) ");
  while (!us_rx_buff()) GetTick_ms ();
  us_putc ((c = us_getc()));
  return (c == 'y' || c == 'Y');
}

char _us_getc (void) {
  while (!us_rx_buff()) GetTick_ms ();
  return us_getc ();
}

/*======================================================================*/
/* OTHER TX/RX ROUTINES                                                 */
/*======================================================================*/
void us_hex_puts (char *s1, char *s2, long num, int d) {
  static char s[20];

  us_puts (s1);
  us_puts (I2H (num, d, s));
  us_puts (s2);
}

typedef enum {
  mode_str,
  mode_hex,
  mode_dec,
} TCharMode;

char *getstr (char *s, int len, TCharMode m) {
  int   i = 0;
  char  c;

  while (1) {
    c = _us_getc();
    if (c == '\n' || c == '\r') {
      s[i++] = '\0';
      break;
    } else if ((i > 0) && (c == '\b')) {
      us_puts ("\b \b");
      i--;
    } else if ((i < len) && (i >= 0) && (c >= ' ') && (c <= '~')) {
      switch (m) {
        case mode_dec:
          if ((c >= '0') && (c <= '9')) {
            s[i++] = (char)c;
            us_putc (c);
          }
          break;
        case mode_hex:
          if (((c >= '0') && (c <= '9')) || ((c >= 'a') && (c <= 'f')) || ((c >= 'A') && (c <= 'F'))) {
            s[i++] = c;
            us_putc (c);
          }
          break;
        default:
          s[i++] = (char)c;
          us_putc (c);
          break;
      }
    } else us_putc ('\a');
  }
  return s;
}

#ifdef _USE_DOWNLOAD
void us_repeatchar (char c, int num) {
  for (; num > 0; num--) us_putc (c);
}
#endif

/*======================================================================*/
/* XMODEM                                                               */
/*======================================================================*/
/*-----XMODEMのパケット処理-------*
  受信側              送信側
         --- NAK -->
         <-- SOH --- (128byte時のブロック開始)
         <--  01 --- (ブロック番号)
         <--  FE --- (ブロック番号を反転させたもの)
         <--  .. --- (実際のデータ 128 bytes)
              ..
         <-- sum --- (checksum)

         --- ACK -->
 *--------------------------------*/
/*-----XMODEM1kのパケット処理-------*
  受信側              送信側
         --- 'C' -->
         <-- STX --- (1024byte時のブロック開始)
         <--  01 --- (ブロック番号)
         <--  FE --- (ブロック番号を反転させたもの)
         <--  .. --- (実際のデータ 1024 bytes)
              ..
         <-- sumH --- (CRC)
         <-- sumL --- (CRC)

         --- ACK -->
 *--------------------------------*/
#define _CNTOVER  (5)

uint8_t xbuff[2048];

bool XmodemRX (void) {

  bool my_put_ack (void) {
    char i = 0;

    for (i = 0; i < _CNTOVER; i++) {
      us_putc (_ACK);  // OK
      if (wait (100, true)) return true;
    }
    return false;
  }

  bool my_get_char (uint8_t *ch, bool f) {
    int cntover = 0;

    if (wait (50, true)) {
      *ch = (uint8_t) _us_getc();
      return true;
    } else {
      if (f) {
        wait (100, false);
        us_rx_purge();
        do {
          us_putc (_NAK);
          wait (100, true);
          if (cntover++ > _CNTOVER) break;
        } while (!us_rx_buff());
      }
      return false;
    }
  }

  int       i = 0;
  uint8_t   head, sum0 = 0, sum1, h0 = 1, h1, h2;
  uint16_t  sum2;
  uint32_t  size = 0, adr = 0, fladdr = FLASH_ADDRESS;
  bool      f;

  while (!us_rx_buff()) {
#if _XMODEM_PACKET_SIZE == 128
    us_putc (_NAK);
#else
    us_putc (_XYC);
#endif
    if (wait (100, true)) break;
    if (i++ > 240) return false;
  }
  for (;;) {
    if (my_get_char (&head, true)) {
      switch (head) {
#if _XMODEM_PACKET_SIZE == 128
        case _SOH:
#else
        case _STX:
#endif
          if (my_get_char (&h1, false)) {
            if (my_get_char (&h2, false)) {
              if (h1 != ((~h2) & 0xff)) {
                wait (50, false);
                us_rx_purge();
                us_putc (_NAK);
              } else {
                if (h1 == h0) {
                  sum2 = 0;
                  f = false;
                  for (i = 0; i < _XMODEM_PACKET_SIZE; i++) {
                    if ((f = my_get_char (&xbuff[adr + i], false))) {
#if _XMODEM_PACKET_SIZE == 128
                      sum2 = (sum2 + xbuff[adr + i]) & 0xff;
#else
                      sum2 = sum2 ^ xbuff[adr + i] << 8;
                      int  j;
                      for (j = 0; j < 8; ++j) {
                        if (sum2 & 0x8000) sum2 = sum2 << 1 ^ 0x1021;
                        else               sum2 = sum2 << 1;
                      }
#endif
                    } else break;
                  }
                  if (f) {
#if _XMODEM_PACKET_SIZE == 128
                    if (my_get_char (&sum1, false))
#else
                    if (my_get_char (&sum0, false) && my_get_char (&sum1, false))
#endif
                    {
                      if (((sum0 << 8) | sum1) == sum2) {
                        adr += _XMODEM_PACKET_SIZE;
                        h0++;
                        if ((adr % sizeof (xbuff)) == 0) {
                          if (FL_WriteFlash (fladdr, sizeof (xbuff), (void *)xbuff)) {
//                        if (true) {
                            fladdr += sizeof (xbuff);
                            adr = 0;
                          } else {
                            us_putc (_CAN);
                            wait (50, false);
                            us_rx_purge();
                            return false;
                          }
                        }
                        size += _XMODEM_PACKET_SIZE;
                        if (!my_put_ack()) {  // OK
                          us_putc (_CAN);
                          wait (50, false);
                          us_rx_purge();
                          return false;
                          break;
                        }
                      } else {
                        wait (50, false);
                        us_rx_purge();
                        us_putc (_NAK);
                      }
                    }
                  } else {
                    wait (50, false);
                    us_rx_purge();
                    us_putc (_NAK);
                  }
                }
              }
            }
          }
          break;
        case _EOT:
          if (adr) {
            if (!FL_WriteFlash (fladdr, adr, (void *)xbuff)) {
              us_putc (_CAN);
              wait (50, false);
              us_rx_purge();
              return false;
            }
          }
          us_rx_purge();
          us_putc (_ACK);
          wait (10, false);
          return true;
          break;
        case _CAN:
          wait (50, false);
          us_rx_purge();
          return false;
          break;
        default:
          wait (50, false);
          us_rx_purge();
          us_putc (_NAK);
          break;
      }
    }
  }
}

int getc2 (int w) {
  wait (w, true);
  if (us_rx_buff()) return _us_getc();
  return -1;
}

#ifdef _USE_DOWNLOAD
int XmodemTX (uint8_t *src, int srcsz) {
  uint16_t  sum2;
  uint8_t   packetno = 1;
  int       packetsize = 0, i, j, c, len = 0, retry = 0;
  bool      crc = false;

  while (!us_rx_buff()) {
    if (wait (100, true)) {
      switch (c = _us_getc()) {
        case _XYC:  // 1kbyte
          crc = true;
          packetsize = 1024;
          break;
        case _NAK:  // 128byte
          crc = false;
          packetsize = 128;
          break;
        case _CAN:  // canceled by remote
          wait (50, false);
          us_rx_purge();
          us_putc (_ACK);
          return -1;
          break;
        default:    // cancel
          wait (50, false);
          us_rx_purge();
          us_putc (_CAN);
          return -1;
          break;
      }
      break;
    }
    if (retry++ > _CNTOVER * 10) return -1;
  }

  for (;;) {
    xbuff[0] = crc ? _STX : _SOH;
    xbuff[1] = packetno;
    xbuff[2] = ~packetno;
    if ((c = srcsz - len) > packetsize) c = packetsize;
    if (c >= 0) {
      memset (&xbuff[3], 0xff, packetsize);
      sum2 = 0;
      for (i = 0; i < packetsize; i++) {
        xbuff[3 + i] = src[len + i];
        if (crc) {
          sum2 = sum2 ^ xbuff[3 + i] << 8;
          for (j = 0; j < 8; ++j) {
            if (sum2 & 0x8000) sum2 = sum2 << 1 ^ 0x1021;
            else               sum2 = sum2 << 1;
          }
        } else sum2 = (sum2 + xbuff[3 + i]) & 0xff;
      }
      if (crc) {
        xbuff[packetsize + 3] = (sum2 >> 8) & 0xFF;
        xbuff[packetsize + 4] = sum2 & 0xFF;
      } else xbuff[packetsize + 3] = sum2;
      for (retry = 0; retry < _CNTOVER; ++retry) {
        us_putsb ((uint8_t *)xbuff, packetsize + 4 + (crc ? 1 : 0));
        if ((c = getc2 (200)) >= 0) {
          switch (c) {
            case _ACK:  // ok
              packetno++;
              len += packetsize;
              retry = _CNTOVER;
              break;
            case _CAN:  // canceled by remote
              wait (50, false);
              us_rx_purge();
              us_putc (_ACK);
              return -1;
              break;
            case _NAK:  // retry
            default:
              wait (50, false);
              us_rx_purge();
              break;
          }
        }
      }
      if (c != _ACK) {
        wait (50, false);
        us_rx_purge();
        us_putc (_CAN);
        return -1;
      }
    } else {
      for (retry = 0; retry < _CNTOVER; retry++) {
        us_putc (_EOT);
        if ((c = getc2 (500)) == _ACK) break;
      }
      us_rx_purge();
      return (c == _ACK) ? len : -5;
    }
  }
}
#endif

/*======================================================================*/
/* DUMP                                                                 */
/*======================================================================*/
void dump (uint32_t addr) {
  uint32_t  i, j;
  uint8_t   *dat = (uint8_t *)addr;
  char      s[80], c;

  while (1) {
    s[0] = ' ';
    for (i = 0; i < 16; i++) {
      I2H ((uint32_t) (dat + (i << 4)), 8, &s[1]);
      s[9] = ':';
      for (j = 0; j < 16; j++) {
        I2H (dat[ (i << 4) + j], 2, &s[10 + j * 3]);
        s[12 + j * 3] = ' ';
      }
      s[57] = ' ';
      for (j = 0; j < 16; j++) {
        c = dat[ (i << 4) + j];
        s[58 + j] = ((0x20 <= c) && (c < 0x7f)) ? c : '.';
      }
      s[74] = '\n';
      s[75] = '\0';
      us_puts (s);
    }
    us_puts ("---([u]up [d]down [ESC]exit)---\n");
    switch (_us_getc()) {
      case 0x1b:
      case 'q':
      case 'Q':
        return;
        break;
      case ' ':
      case 'u':
      case 'U':
      case '\r':
        dat += 256;
        break;
      case 'd':
      case 'D':
        dat -= 256;
        break;
    }
#ifdef LED
    duty = 100;
#endif
  }
}

/*======================================================================*/
/* REPLACE RESET RESETEXCEPTION HANDLER                                 */
/*======================================================================*/
typedef void (*fp) (void) ;

void ResetExceptionDispatch_Top (void) {
  uint32_t  *FWBOOT = (uint32_t *)FLASH_ADDRESS;
  // Branch to user program
#if 1
//  if ((ResetType () != 3) && (ResetType () != 4) && (*FWBOOT != 0xffffffff)) {
  if ((ResetType() == 0) && (*FWBOOT != 0xffffffff)) {
    fp UserProgram = (fp)FLASH_ADDRESS;
    UserProgram();
  }
#endif
}

/*======================================================================*/
/* MAIN                                                                 */
/*======================================================================*/
void main (void) {

  void UPG (void) {
    fp UserProgram = (fp)FLASH_ADDRESS;
    DeInit();
//    MC.RCR = 1;
    UserProgram();
  }

  int32_t   i;
#ifdef _USE_DOWNLOAD
  int32_t   j;
#endif
  char      s[10], c = 3, ch = '\0', bs[] = "\33[0D";
  bool      t;
  uint32_t  *FWBOOT = (uint32_t *)FLASH_ADDRESS;
  const char *resetmess[6] = {
    "\n PowerUp Reset",
    "",
    "\n Watchdog Reset",
    "\n Software Reset",
    "\n User Reset",
    "\n BOD Reset"
  };

  Init();

#if defined (__USB__)
  i = GetTick_ms() + 2000;
  do rx_int(); while (i > GetTick_ms());
  while (!port_opened (&pCDC)) rx_int();
#endif

  us_puts (resetmess[ResetType()]);
  us_puts (mess[0]);

  for (;;) {
    us_rx_purge();
    us_puts ("\n>");
    us_putc (ch = _us_getc());

#if defined (_MON_LED)
    if (PIOA.PDSR & _MON_LED) PIOA.CODR = _MON_LED;
    else                      PIOA.SODR = _MON_LED;
#endif
    switch (tolower ((int)ch)) {
      case 'w':
        if (AreYouSure ("\n START UPLOAD...")) us_puts (XmodemRX() ? "\n <SUCCESS>" : "\n <ERROR>");
        break;
#ifdef _USE_DOWNLOAD
      case 'r':
        us_puts ("\n START ADDRESS:$");
        if ((i = H2I (getstr (s, 8, mode_hex))) < 0) i = FLASH_ADDRESS;
        bs[2] = strlen (s) + 0x30;
        us_hex_puts (bs, "\n SiZE:$", i, 8);
        if ((j = H2I (getstr (s, 8, mode_hex))) < 0) j = FLASH_BYTE_SIZE;
        bs[2] = strlen (s) + 0x30;
        us_hex_puts (bs, "\n START DOWNLOAD...", j, 8);
        if (AreYouSure ("")) us_puts (XmodemTX ((uint8_t *) (uint32_t)i, j) ? "\n <SUCCESS>" : "\n <ERROR>");
        break;
#endif
      case 'g':
        if (*FWBOOT != 0xffffffff) {
          us_puts ("\n GO !!\n");
          wait (20, true);
          UPG();
        } else us_puts ("\n FLASH IS EMPTY");
        break;
      case 'i':
        us_puts ("\n MC.EFC0.FSR:$");
        us_puts (I2H (MC.EFC0.FSR, 8, s));
#if defined (__AT91SAM7S512__) | defined (__AT91SAM7X512__) | defined (__AT91SAM7SE512__)
        us_puts ("\n MC.EFC1.FSR:$");
        us_puts (I2H (MC.EFC1.FSR, 8, s));
#endif
        if (*FWBOOT == 0xffffffff) us_puts ("\n FLASH IS EMPTY");
        break;
      case 'e':
        if (AreYouSure ("\n ERASE ALL FLASH")) us_puts (EraseFlashAll() ? "\n <SUCCESS>" : "\n <ERROR>");
        break;
      case 'n':
      case 'c':
        t = (ch == 'n');
        us_puts (t ? "\n SET GPNVM Bit" : "\n CLEAR GPNVM Bit");
#if defined (__AT91SAM7S32__) |  defined (__AT91SAM7S64__) | defined (__AT91SAM7S128__) | defined (__AT91SAM7S256__) | defined (__AT91SAM7S512__) | defined (__AT91SAM7SE256__) | defined (__AT91SAM7SE512__)
        us_puts ("\n SELECT BIT NO.(0..1) ");
        us_putc (c = _us_getc());
        switch (c) {
          case '0':
          case '1':
            t ? FL_NVMSet (c - '0') : FL_NVMClear (c - '0');
            us_puts ("\n COMPLETED.");
            break;
          default:
            us_puts ("\n CANCELED.");
            break;
        }
#elif defined (__AT91SAM7X128__) | defined (__AT91SAM7X256__) | defined (__AT91SAM7X512__) | defined (__AT91SAM7SE256__) | defined (__AT91SAM7SE512__)
        us_puts ("\n SELECT BIT NO.(0..2) ");
        us_putc (c = _us_getc());
        switch (c) {
          case '0':
          case '1':
          case '2':
            t ? FL_NVMSet (c - '0') : FL_NVMClear (c - '0');
            us_puts ("\n COMPLETED.");
            break;
          default:
            us_puts ("\n CANCELED.");
            break;
        }
#endif
        break;
      case 'l':
      case 'u':
        t = (ch == 'l');
        if (AreYouSure (t ? "\n SET LOCK Bit" : "\n CLEAR LOCK Bit")) {
#if defined (__AT91SAM7S512__) | defined (__AT91SAM7X512__) | defined (__AT91SAM7SE512__)
          for (i = 0; i < FLASH_PAGE_NB * 2; i += FLASH_PAGE_LOCK) {
#else
          for (i = 0; i < FLASH_PAGE_NB; i += FLASH_PAGE_LOCK) {
#endif
            us_puts ("\n PAGE:");
            us_puts (I2D (i, 4, s));
            t ? FL_Flashlock (i) : FL_FlashUnlock (i);
          }
          us_puts ("\n COMPLETED.");
        }
        else us_puts ("\n CANCELED.");
        break;
      case 'x':
        wait (20, false);
        DeInit();
        while (1) RSTC.CR = 0xA5000000 | _RSTC_PROCRST | _RSTC_PERRST | _RSTC_EXTRST;
        break;
#if defined(_USE_SECURITYBIT)
      case 's':
        if (AreYouSure ("\n SET SECURITY Bit")) FL_SetSecurity();
        break;
#endif
      case 'd':
        us_puts ("\n START ADDRESS:$");
        if ((i = H2I (getstr (s, 8, mode_hex))) < 0) i = FLASH_ADDRESS;
        bs[2] = strlen (s) + 0x30;
        us_hex_puts (bs, "\n", i, 8);
        dump (i);
        break;
      case '?':
        us_puts ((char *)mess[0]);
        us_puts ((char *)mess[1]);
        break;
    }
  }
}
